<?php
include('zip.lib.php');

define("EOT_ROW_NOT_FOUND", 0);
define("EOT_ROW_IS_PROCESSING", 1);
define("EOT_ROW_IS_COMPLETED", 2);


class OODocFiller {
    // Parameters (to change use only SetParameter)
    var $TemplateFileName;
    var $OutFileName;
    var $FieldPrefix;
    var $HeaderPartName;
    var $FooterPartName;
    var $NamePartDelimeter;
    var $RowElement;
    var $EndOfTableMarker;
    var $Encoding;

    // Working vars
    var $HeaderPart;
    var $FooterPart;
    var $Rows;

    var $TemplateXML;
    var $OutputXML;
    var $OutputZip;

    var $DefaultRow;
    var $RowElementLevel;
    var $TRowName;
    var $TRowBuffer;
    var $EOTRowStage;

    var $Error;


    function OODocFiller($pTemplateFileName){
        $this->TemplateFileName = $pTemplateFileName;

        // Defaults
        $this->OutFileName = 'out.sxw';
        $this->FieldPrefix = 'fl';
        $this->HeaderPartName = 'hd';
        $this->FooterPartName = 'ft';
        $this->NamePartDelimeter = preg_quote('.');
        $this->RowElement = 'text:p';
        $this->EndOfTableMarker = preg_quote('EOT');
        $this->Encoding = 'WINDOWS-1251';

        $this->EOTRowStage = EOT_ROW_NOT_FOUND;
		$this->RowElementLevel = 0;
        $this->Error = "";
        $this->OutputXML = "";
    }

    function SetParameter($ParName, $ParValue){
        if($ParName == "NamePartDelimeter" || $ParName == "EndOfTableMarker"){
            $ParValue = preg_quote($ParValue);
        }
        $this->$ParName = $ParValue;
    }
    /* SetParameter */

    function &GetHeaderPart(){
        $this->HeaderPart = new OODocFillerMember($this, $this->HeaderPartName);
        $this->DefaultRow = &$this->HeaderPart;
		if(func_num_args()){
		    $this->HeaderPart->BindFields(func_get_args());
		}
        return $this->HeaderPart;
    }

    function &GetFooterPart(){
        $this->FooterPart = new OODocFillerMember($this, $this->FooterPartName);
		if(func_num_args()){
		    $this->FooterPart->BindFields(func_get_args());
		}
        return $this->FooterPart;
    }

    function &GetNewRow(){
	    $RowName = func_get_arg(0);
        $this->Rows[$RowName] = new OODocFillerMember($this, $RowName);
		if(func_num_args() > 1){
		    $pararr = func_get_args();
		    array_shift($pararr);
			$this->Rows[$RowName]->BindFields($pararr);
		}
        return $this->Rows[$RowName];
    }

    function OpenTemplate(){
        $this->OutputZip = new zipfile();
        if(!($zip = zip_open($this->TemplateFileName))){
            $this->Error .= "Error during openinig template file [".$this->TemplateFileName."]\n";
            return false;
        }
        while($zip_entry = zip_read($zip)) {
            if(!zip_entry_open($zip, $zip_entry, "r")){
                $this->Error .= "Error inside template file [".$this->TemplateFileName."]\n";
                return false;
            }
            $buf = zip_entry_read($zip_entry, zip_entry_filesize($zip_entry));
            if(zip_entry_name($zip_entry) == "content.xml"){
                $this->TemplateXML = $buf;
            }
            else{
                $this->OutputZip->addFile($buf, zip_entry_name($zip_entry));
            }
            zip_entry_close($zip_entry);
        }
        zip_close($zip);
        return true;
    }
    /* OpenTemplate */

    function ProcessTemplate(){
        if(!$this->OpenTemplate())
           return false;

        $xml_parser = xml_parser_create();
        xml_set_object($xml_parser, $this);
		xml_parser_set_option($xml_parser, XML_OPTION_CASE_FOLDING, false);
        xml_set_element_handler($xml_parser, "StartH", "EndH");
        xml_set_character_data_handler($xml_parser, "CharH");
        xml_set_default_handler($xml_parser, "DefaultH");

        // Now go to parse
        if (!xml_parse($xml_parser, $this->TemplateXML)){
           $this->Error .= sprintf("XML error: %s at line %d\n",
                   xml_error_string(xml_get_error_code($xml_parser)),
                   xml_get_current_line_number($xml_parser));
           return false;
        }
        return true;
    }
    /* ProcessTemplate */

    function StartH($parser, $el, $atts){
        $Element = "<$el";
        while(list($key, $val) = each($atts)) {
            $Element .= " $key=\"".xmlquot($val)."\"";
        }
        $Element .= ">";


        if($el == $this->RowElement){ # It can be a start of user table row
            $this->RowElementLevel += 1;
            $this->TRowBuffer .= $Element;
        }
        else{
            if($this->RowElementLevel){ # Store for further analysis
                $this->TRowBuffer .= $Element;
            }
            else{ # Just print out
                $this->DefaultRow->RowBody .= $Element;
            }
        }
    }
    /* StartH */

    function EndH($parser, $el){
        $Element = "</$el>";

        if($el == $this->RowElement){ # It can be a end of user table row
            $this->TRowBuffer .= $Element;
            if($this->RowElementLevel > 1){ # The Elemet is incomplete
                $this->RowElementLevel -= 1;
            }
            else{
                if($this->TRowName){ # Store User table row
                    if($this->EOTRowStage == EOT_ROW_IS_PROCESSING){
                        $this->EOTRowStage = EOT_ROW_IS_COMPLETED; # We just throw out End of table row
                        $this->DefaultRow = &$this->FooterPart;    # And switch Default row to footer
                    }
                    else{
                        $this->Rows[$this->TRowName]->RowBody =
                            $this->TRowBuffer;
                    }
                    $this->TRowName = "";
                }
                else{ # Print out not a users Row
                    $this->DefaultRow->RowBody .= $this->TRowBuffer;
                }
                $this->TRowBuffer = "";
                $this->RowElementLevel = 0;
            }
        }
        else{ # It is usual, not a user row element
            if($this->RowElementLevel){ # Store for further using
                $this->TRowBuffer .= $Element;
            }
            else{ # Just print out
                $this->DefaultRow->RowBody .= $Element;
            }
        }
    }
    /* EndH */

    function CharH($parser, $str){
        $templ = '@'.$this->FieldPrefix.$this->NamePartDelimeter.'(\w+)'.
            $this->NamePartDelimeter.'(\w+)'.'@';
        $templEOT = '@'.$this->EndOfTableMarker.'@';

        if(!($this->TRowName)){ // User's row is not started
            if(preg_match($templ, $str, $matches)){ // There is an external field
                if(!($matches[1] == $this->HeaderPartName || $matches[1] == $this->FooterPartName)){ // The field from usr's row
                    if($this->RowElementLevel != 0){
                        // Current row is user's
                        $this->TRowName = $matches[1];
                    }
                    else{
                        $this->Error .= "Field of users's row [".$matches[0]."] is outside XML row elment\n";
                    }
                }
            }
            elseif(preg_match($templEOT, $str, $matches)){
                if($this->EOTRowStage == EOT_ROW_NOT_FOUND){
                    $this->EOTRowStage = EOT_ROW_IS_PROCESSING;
                    $this->TRowName = " \t\n\t "; # We have to collect all EOT row data
                }
                else{
                    $this->Error .= "EOT marker found in the wrong place or marker is not single\n";
                }
            }

            // Print out
            if($this->RowElementLevel){
                $this->TRowBuffer .= $str;
            }
            else{
                $this->DefaultRow->RowBody .= $str;
            }
        }
        else{  // Data from user's row
            $this->TRowBuffer .= $str;
        }
    }
    /* CharH */

    function DefaultH($parser, $str){
	    $this->DefaultRow->RowBody .= $str; //$str
    }
    /* DefaultH */

    function GetBinaryResult(){
        $this->OutputZip->addFile($this->OutputXML, 'content.xml');
        return $this->OutputZip->file();
    }
    /* GetBinaryResult */
}


class OODocFillerMember {
    var $FillerRef;
    var $RowName;

    var $RowBody;
    var $FieldsList;
    var $FieldsData;

    function OODocFillerMember(&$pFillerRef, $pRowName){
        $this->FillerRef = &$pFillerRef;
        $this->RowName = $pRowName;
        $this->RowBody = "";
    }

    function BindFields(){
   	    if(is_array(func_get_arg(0))){
            $this->FieldsList = func_get_arg(0);
        }
        else{
            $this->FieldsList = func_get_args();
        }

        for($i = 0; $i < count($this->FieldsList); ++$i){
            $this->FieldsList[$i] = '@'.$this->FillerRef->FieldPrefix.$this->FillerRef->NamePartDelimeter.$this->RowName.
                $this->FillerRef->NamePartDelimeter.$this->FieldsList[$i].'@';
        }
    }

    function SetFieldsData(){
        if(is_array(func_get_arg(0))){
            $this->FieldsData = func_get_arg(0);
        }
        else{
            $this->FieldsData = func_get_args();
        }

        for($i = 0; $i < count($this->FieldsData); ++$i){
            $this->FieldsData[$i] = iconv($this->FillerRef->Encoding, "UTF-8", $this->FieldsData[$i]);
        }
    }
    /* SetFieldsData */

    function Fill(){
        if(func_num_args()){
            $this->SetFieldsData(func_get_args());
        }

        if(($this->RowName == $this->FillerRef->HeaderPartName) && (!$this->RowBody)){
            if(!$this->FillerRef->ProcessTemplate()){
                return false;
            }
        }
    
        $this->FillerRef->OutputXML .= preg_replace($this->FieldsList, $this->FieldsData, $this->RowBody);
        return true;
    }
    /* PrintRow */

}


function xmlquot($str) {
    $pattern = array('@&@', '@<@', '@>@', '@"@', '@\'@');
    $replacement = array('&amp;', '&lt;', '&gt;', '&quot;', '&apos;');

    return preg_replace($pattern, $replacement, $str);
}
/* xmlquot */
?>
